"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpDispatcherServiceClient = void 0;
const axios_1 = require("./axios");
const logger = require("../../../log");
class HttpDispatcherServiceClient {
    constructor(baseUrl) {
        this.version = '2022-12-01~experimental';
        this.baseUrl = baseUrl;
    }
    async createConnection(params, data) {
        try {
            const path = `/hidden/broker/${params.hashedBrokerToken}/connections/${params.brokerClientId}`;
            const response = await axios_1.axiosInstance.post(path, {
                data: {
                    attributes: {
                        deployment_location: data.deployment_location,
                    },
                },
            }, {
                baseURL: this.baseUrl,
                headers: {
                    'Content-type': 'application/vnd.api+json',
                },
                params: { version: this.version },
                validateStatus: () => true,
            });
            const apiResponse = response.data.data;
            if (!(apiResponse === null || apiResponse === void 0 ? void 0 : apiResponse.attributes)) {
                logger.trace({ apiResponse }, 'Unexpected Connection Allocation Server response');
                throw new Error('Unexpected connection allocation server response');
            }
            const snykRequestId = response.headers['snyk-request-id'] || '';
            logger.trace({ snykRequestId, apiResponse }, 'createConnection API response');
            const serverId = apiResponse.attributes.server_id;
            return serverId;
        }
        catch (err) {
            logger.trace({ err }, 'Error getting connection allocation.');
            throw new Error('Error getting connection allocation.');
        }
    }
}
exports.HttpDispatcherServiceClient = HttpDispatcherServiceClient;
//# sourceMappingURL=api.js.map