"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getServerId = exports.highAvailabilityModeEnabled = exports.defaultBrokerDispatcherBaseUrl = void 0;
const crypto = require("crypto");
const logger = require("../../log");
const api_1 = require("./client/api");
const dispatcher_service_1 = require("./dispatcher-service");
exports.defaultBrokerDispatcherBaseUrl = 'https://api.snyk.io';
function highAvailabilityModeEnabled(config) {
    const haConfig = getHAConfig(config);
    logger.info({ enabled: haConfig.BROKER_HA_MODE_ENABLED }, 'checking for HA mode');
    return haConfig.BROKER_HA_MODE_ENABLED;
}
exports.highAvailabilityModeEnabled = highAvailabilityModeEnabled;
async function getServerId(config, brokerToken, brokerClientId) {
    const haConfig = getHAConfig(config);
    const baseUrl = haConfig.BROKER_DISPATCHER_BASE_URL || exports.defaultBrokerDispatcherBaseUrl;
    const client = new api_1.HttpDispatcherServiceClient(baseUrl);
    const token = hash(config.BROKER_TOKEN);
    const maxRetries = 30;
    for (let attempt = 0; attempt <= maxRetries; attempt++) {
        try {
            return await (0, dispatcher_service_1.getServerIdFromDispatcher)(client, {
                brokerClientId: brokerClientId,
                hashedBrokerToken: token,
            });
        }
        catch (err) {
            const timeout = 2 ** attempt * 100;
            logger.warn({ attempt, timeout }, `waiting for ${timeout}ms before next Broker Dispatcher API call`);
            Atomics.wait(new Int32Array(new SharedArrayBuffer(4)), 0, 0, timeout);
        }
    }
    return null;
}
exports.getServerId = getServerId;
function getHAConfig(config) {
    return config;
}
function hash(token) {
    const shasum = crypto.createHash('sha256');
    shasum.update(token);
    return shasum.digest('hex');
}
//# sourceMappingURL=index.js.map