const { Gauge, Histogram, Counter } = require('prom-client');
const socketConnectionGauge = new Gauge({
    name: 'broker_socket_connection_total',
    help: 'Number of active socket connections',
});
const responseSizeHistogram = new Histogram({
    name: 'broker_response_size_bytes',
    help: 'The size of broker server responses in bytes',
    labelNames: ['isStreaming'],
    buckets: [
        1024,
        5120,
        10240,
        25600,
        51200,
        102400,
        512000,
        1048576,
        10485760,
        20971520, // 20mb 'maxLength' in socket.js
    ],
});
const unableToSizeResponseCounter = new Counter({
    name: 'broker_unable_to_size_response_count',
    help: 'A count of the number of times broker server was unable to size a response',
});
function incrementSocketConnectionGauge() {
    socketConnectionGauge.inc(1);
}
function decrementSocketConnectionGauge() {
    socketConnectionGauge.dec(1);
}
function observeResponseSize({ bytes, isStreaming }) {
    responseSizeHistogram.observe({ isStreaming }, bytes);
}
function incrementUnableToSizeResponse() {
    unableToSizeResponseCounter.inc(1);
}
module.exports = {
    incrementSocketConnectionGauge,
    decrementSocketConnectionGauge,
    observeResponseSize,
    incrementUnableToSizeResponse,
};
//# sourceMappingURL=metrics.js.map